import os, sys, re
import datetime
import dateparser
import logging

from bs4 import BeautifulSoup
from urllib.parse import unquote

from RAAspotter import RAAspotter

logger = logging.getLogger(__name__)

class RAAspotter_pref59(RAAspotter):

  # Config
  __HOST       = 'https://www.nord.gouv.fr'
  __RAA_PAGE   = {'2024': f'{__HOST}/Publications/Recueils-des-actes-administratifs/RAA-du-departement-du-Nord/2024',
                  '2023': f'{__HOST}/Publications/Recueils-des-actes-administratifs/RAA-du-departement-du-Nord/2023',
                  '2022': f'{__HOST}/Publications/Recueils-des-actes-administratifs/RAA-du-departement-du-Nord/2022',
                  '2021': f'{__HOST}/Publications/Recueils-des-actes-administratifs/RAA-du-departement-du-Nord/2021',
                  '2020': f'{__HOST}/Publications/Recueils-des-actes-administratifs/RAA-du-departement-du-Nord/2020',
                  '2019': f'{__HOST}/Publications/Recueils-des-actes-administratifs/RAA-du-departement-du-Nord/2019'}
  __USER_AGENT = 'Mozilla/5.0 (X11; Linux x86_64; rv:109.0) Gecko/20100101 Firefox/115.0'
  full_name = 'Préfecture du Nord'
  short_code = 'pref59'

  def __init__(self, data_dir):
    super().__init__(data_dir, self.__USER_AGENT)
    self.enable_tor(20)

  def get_raa(self, keywords):
    self.print_output('RAAspotter_pref59')
    self.print_output(f'Termes recherchés: {keywords}')
    self.print_output('')

    pages_to_parse = []
    if self.not_before.year <= 2024:
      pages_to_parse.append(self.__RAA_PAGE['2024'])
    if self.not_before.year <= 2023:
      pages_to_parse.append(self.__RAA_PAGE['2023'])
    if self.not_before.year <= 2022:
      pages_to_parse.append(self.__RAA_PAGE['2022'])
    if self.not_before.year <= 2021:
      pages_to_parse.append(self.__RAA_PAGE['2021'])
    if self.not_before.year <= 2020:
      pages_to_parse.append(self.__RAA_PAGE['2020'])
    if self.not_before.year <= 2019:
      pages_to_parse.append(self.__RAA_PAGE['2019'])

    for raa_page in pages_to_parse:
      page_content = self.get_page(raa_page, 'get').content
      sub_pages = self.get_sub_pages(page_content, "div.fr-card__body div.fr-card__content h2.fr-card__title a", self.__HOST, True)
      for sub_page in sub_pages[::-1]:
        sub_page_content = self.get_page(sub_page['url'], 'get').content
        sub_raa_elements = self.get_raa_elements(sub_page_content)
        self.parse_raa(sub_raa_elements, keywords.split(','))
    self.mailer()

  def get_raa_elements(self, page_content):
    elements = []
    # On charge le parser
    soup = BeautifulSoup(page_content, 'html.parser')

    # On récupère chaque balise a
    for a in soup.select('a.fr-link.fr-link--download'):
      if a.get('href') and a['href'].endswith('.pdf'):
          if a['href'].startswith('/'):
            url = f"{self.__HOST}{a['href']}"
          else:
            url = a['href']

          url      = unquote(url)
          name     = a.find('span').previous_sibling.replace('Télécharger ', '').strip()
          date     = datetime.datetime.strptime(a.find('span').get_text().split(' - ')[-1].strip(), '%d/%m/%Y')
          filename = url.split('/')[-1]

          raa = RAAspotter.RAA(url, date, name, filename)
          elements.append(raa)
    return elements
